@@ -64,14 +64,7 @@
 
 /*
  * Structures for I/O stats.
- * The number of structures allocated corresponds to the number of devices
- * present in the system, plus a preallocation number to handle those
- * that can be registered dynamically.
- * The number of devices is found by using /sys filesystem (if mounted).
- * For each io_stats structure allocated corresponds a io_hdr_stats structure.
- * A io_stats structure is considered as unused or "free" (containing no stats
- * for a particular device) if the 'major' field of the io_hdr_stats
- * structure is set to 0.
+ * These are now dynamically allocated.
  */
 struct io_stats {
 	/* # of sectors read */
@@ -92,12 +85,16 @@ struct io_stats {
 	unsigned long dc_ios		__attribute__ ((packed));
 	/* # of discard requests merged */
 	unsigned long dc_merges		__attribute__ ((packed));
+	/* # of flush requests issued to the device */
+	unsigned long fl_ios		__attribute__ ((packed));
 	/* Time of read requests in queue */
 	unsigned int  rd_ticks		__attribute__ ((packed));
 	/* Time of write requests in queue */
 	unsigned int  wr_ticks		__attribute__ ((packed));
 	/* Time of discard requests in queue */
 	unsigned int  dc_ticks		__attribute__ ((packed));
+	/* Time of flush requests in queue */
+	unsigned int  fl_ticks		__attribute__ ((packed));
 	/* # of I/Os in progress */
 	unsigned int  ios_pgr		__attribute__ ((packed));
 	/* # of ticks total (for this device) for I/O */
@@ -133,6 +130,8 @@ struct ext_io_stats {
 	double w_await;
 	/* d_await */
 	double d_await;
+	/* f_await */
+	double f_await;
 	/* rsec/s */
 	double rsectors;
 	/* wsec/s */
