@@ -84,6 +84,9 @@ int *cpu_per_node;
  */
 int *cpu2node;
 
+/* CPU topology */
+struct cpu_topology *st_cpu_topology;
+
 struct tm mp_tstamp[3];
 
 /* Activity flag */
@@ -238,6 +241,11 @@ void salloc_mp_struct(int nr_cpus)
 		perror("malloc");
 		exit(4);
 	}
+
+	if ((st_cpu_topology = (struct cpu_topology *) malloc(sizeof(struct cpu_topology) * nr_cpus)) == NULL) {
+		perror("malloc");
+		exit(4);
+	}
 }
 
 /*
@@ -331,6 +339,65 @@ int get_node_placement(int nr_cpus, int cpu_per_node[], int cpu2node[])
 	return hi_node_nr;
 }
 
+/*
+ ***************************************************************************
+ * Read system logical topology: Socket number for each logical core is read
+ * from the /sys/devices/system/cpu/cpu{N}/topology/physical_package_id file,
+ * and the logical core id number is the first number read from the
+ * /sys/devices/system/cpu/cpu{N}/topology/thread_siblings_list file.
+ * Don't use /sys/devices/system/cpu/cpu{N}/topology/core_id as this is the
+ * physical core id (seems to be different from the number displayed by lscpu).
+ *
+ * IN:
+ * @nr_cpus	Number of CPU on this machine.
+ * @cpu_topo	Structures where socket and core id numbers will be saved.
+ *
+ * OUT:
+ * @cpu_topo	Structures where socket and core id numbers have been saved.
+ ***************************************************************************
+ */
+void read_topology(int nr_cpus, struct cpu_topology *cpu_topo)
+{
+	struct cpu_topology *cpu_topo_i;
+	FILE *fp;
+	char filename[MAX_PF_NAME];
+	int cpu, rc;
+
+	/* Init system topology */
+	memset(st_cpu_topology, 0, sizeof(struct cpu_topology) * nr_cpus);
+
+	for (cpu = 0; cpu < nr_cpus; cpu++) {
+
+		cpu_topo_i = cpu_topo + cpu;
+
+		/* Read current CPU's socket number */
+		snprintf(filename, MAX_PF_NAME, "%s/cpu%d/%s", SYSFS_DEVCPU, cpu, PHYS_PACK_ID);
+		filename[MAX_PF_NAME - 1] = '\0';
+
+		if ((fp = fopen(filename, "r")) != NULL) {
+			rc = fscanf(fp, "%d", &cpu_topo_i->phys_package_id);
+			fclose(fp);
+
+			if (rc < 1) {
+				cpu_topo_i->phys_package_id = -1;
+			}
+		}
+
+		/* Read current CPU's logical core id number */
+		snprintf(filename, MAX_PF_NAME, "%s/cpu%d/%s", SYSFS_DEVCPU, cpu, THREAD_SBL_LST);
+		filename[MAX_PF_NAME - 1] = '\0';
+
+		if ((fp = fopen(filename, "r")) != NULL) {
+			rc = fscanf(fp, "%d", &cpu_topo_i->logical_core_id);
+			fclose(fp);
+
+			if (rc < 1) {
+				cpu_topo_i->logical_core_id = -1;
+			}
+		}
+	}
+}
+
 /*
  ***************************************************************************
  * Compute node statistics: Split CPU statistics among nodes.
@@ -554,11 +621,15 @@ void write_plain_cpu_stats(int dis, unsigned long long deltot_jiffies, int prev,
 {
 	int i;
 	struct stats_cpu *scc, *scp;
+	struct cpu_topology *cpu_topo_i;
 
 	if (dis) {
-		printf("\n%-11s  CPU    %%usr   %%nice    %%sys %%iowait    %%irq   "
-		       "%%soft  %%steal  %%guest  %%gnice   %%idle\n",
-		       prev_string);
+		printf("\n%-11s  CPU", prev_string);
+		if (DISPLAY_TOPOLOGY(flags)) {
+			printf(" CORE SOCK NODE");
+		}
+		printf("    %%usr   %%nice    %%sys %%iowait    %%irq   "
+		       "%%soft  %%steal  %%guest  %%gnice   %%idle\n");
 	}
 
 	/*
@@ -580,10 +651,21 @@ void write_plain_cpu_stats(int dis, unsigned long long deltot_jiffies, int prev,
 		if (i == 0) {
 			/* This is CPU "all" */
 			cprintf_in(IS_STR, " %s", " all", 0);
+
+			if (DISPLAY_TOPOLOGY(flags)) {
+				printf("               ");
+			}
 		}
 		else {
 			cprintf_in(IS_INT, " %4d", "", i - 1);
 
+			if (DISPLAY_TOPOLOGY(flags)) {
+				cpu_topo_i = st_cpu_topology + i - 1;
+				cprintf_in(IS_INT, " %4d", "", cpu_topo_i->logical_core_id);
+				cprintf_in(IS_INT, " %4d", "", cpu_topo_i->phys_package_id);
+				cprintf_in(IS_INT, " %4d", "", cpu2node[i - 1]);
+			}
+
 			/* Recalculate itv for current proc */
 			deltot_jiffies = get_per_cpu_interval(scc, scp);
 
@@ -651,8 +733,9 @@ void write_json_cpu_stats(int tab, unsigned long long deltot_jiffies, int prev,
 			  unsigned char offline_cpu_bitmap[])
 {
 	int i, next = FALSE;
-	char cpu_name[16];
+	char cpu_name[16], topology[1024] = "";
 	struct stats_cpu *scc, *scp;
+	struct cpu_topology *cpu_topo_i;
 
 	xprintf(tab++, "\"cpu-load\": [");
 
@@ -679,11 +762,23 @@ void write_json_cpu_stats(int tab, unsigned long long deltot_jiffies, int prev,
 			/* This is CPU "all" */
 			strcpy(cpu_name, "all");
 
+			if (DISPLAY_TOPOLOGY(flags)) {
+				snprintf(topology, 1024,
+					 ", \"core\": \"\", \"socket\": \"\", \"node\": \"\"");
+			}
+
 		}
 		else {
 			snprintf(cpu_name, 16, "%d", i - 1);
 			cpu_name[15] = '\0';
 
+			if (DISPLAY_TOPOLOGY(flags)) {
+				cpu_topo_i = st_cpu_topology + i - 1;
+				snprintf(topology, 1024,
+					 ", \"core\": \"%d\", \"socket\": \"%d\", \"node\": \"%d\"",
+					 cpu_topo_i->logical_core_id, cpu_topo_i->phys_package_id, cpu2node[i - 1]);
+			}
+
 			/* Recalculate itv for current proc */
 			deltot_jiffies = get_per_cpu_interval(scc, scp);
 
@@ -692,19 +787,20 @@ void write_json_cpu_stats(int tab, unsigned long long deltot_jiffies, int prev,
 				 * If the CPU is tickless then there is no change in CPU values
 				 * but the sum of values is not zero.
 				 */
-				xprintf0(tab, "{\"cpu\": \"%d\", \"usr\": 0.00, \"nice\": 0.00, "
+				xprintf0(tab, "{\"cpu\": \"%d\"%s, \"usr\": 0.00, \"nice\": 0.00, "
 					 "\"sys\": 0.00, \"iowait\": 0.00, \"irq\": 0.00, "
 					 "\"soft\": 0.00, \"steal\": 0.00, \"guest\": 0.00, "
-					 "\"gnice\": 0.00, \"idle\": 100.00}", i - 1);
+					 "\"gnice\": 0.00, \"idle\": 100.00}", i - 1, topology);
 				printf("\n");
 
 				continue;
 			}
 		}
 
-		xprintf0(tab, "{\"cpu\": \"%s\", \"usr\": %.2f, \"nice\": %.2f, \"sys\": %.2f, "
+		xprintf0(tab, "{\"cpu\": \"%s\"%s, \"usr\": %.2f, \"nice\": %.2f, \"sys\": %.2f, "
 			 "\"iowait\": %.2f, \"irq\": %.2f, \"soft\": %.2f, \"steal\": %.2f, "
-			 "\"guest\": %.2f, \"gnice\": %.2f, \"idle\": %.2f}", cpu_name,
+			 "\"guest\": %.2f, \"gnice\": %.2f, \"idle\": %.2f}",
+			 cpu_name, topology,
 			 (scc->cpu_user - scc->cpu_guest) < (scp->cpu_user - scp->cpu_guest) ?
 			 0.0 :
 			 ll_sp_value(scp->cpu_user - scp->cpu_guest,
@@ -1329,7 +1425,7 @@ void write_plain_irqcpu_stats(struct stats_irqcpu *st_ic[], int ic_nr, int dis,
 		 * CPU must have been explicitly selected using option -P,
 		 * else we display every CPU.
 		 */
-		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_P_OPTION(flags))
+		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_OPTION_P(flags))
 			continue;
 
 		if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
@@ -1429,7 +1525,7 @@ void write_json_irqcpu_stats(int tab, struct stats_irqcpu *st_ic[], int ic_nr,
 		 * CPU must have been explicitly selected using option -P,
 		 * else we display every CPU.
 		 */
-		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_P_OPTION(flags))
+		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_OPTION_P(flags))
 			continue;
 
 		if ((scc->cpu_user    + scc->cpu_nice + scc->cpu_sys   +
@@ -1824,6 +1920,11 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 		}
 	}
 
+	/* Read system topology */
+	if (DISPLAY_CPU(actflags) && DISPLAY_TOPOLOGY(flags)) {
+		read_topology(cpu_nr, st_cpu_topology);
+	}
+
 	/*
 	 * Read total number of interrupts received among all CPU.
 	 * (this is the first value on the line "intr:" in the /proc/stat file).
@@ -1906,6 +2007,11 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 		read_uptime(&(uptime_cs[curr]));
 		read_stat_cpu(st_cpu[curr], cpu_nr + 1);
 
+		/* Read system topology */
+		if (DISPLAY_CPU(actflags) && DISPLAY_TOPOLOGY(flags)) {
+			read_topology(cpu_nr, st_cpu_topology);
+		}
+
 		/* Read total number of interrupts received among all CPU */
 		if (DISPLAY_IRQ_SUM(actflags)) {
 			read_stat_irq(st_irq[curr], 1);
@@ -2059,7 +2165,7 @@ int main(int argc, char **argv)
 				usage(argv[0]);
 			}
 			if (node_nr >= 0) {
-				flags |= F_N_OPTION;
+				flags |= F_OPTION_N;
 				actflags |= M_D_NODE;
 				actset = TRUE;
 				dis_hdr = 9;
@@ -2074,7 +2180,7 @@ int main(int argc, char **argv)
 			if (!argv[++opt]) {
 				usage(argv[0]);
 			}
-			flags |= F_P_OPTION;
+			flags |= F_OPTION_P;
 			dis_hdr = 9;
 
 			if (parse_values(argv[opt], cpu_bitmap, cpu_nr, K_LOWERALL)) {
@@ -2088,16 +2194,12 @@ int main(int argc, char **argv)
 				switch (*(argv[opt] + i)) {
 
 				case 'A':
+					flags |= F_OPTION_A;
 					actflags |= M_D_CPU + M_D_IRQ_SUM + M_D_IRQ_CPU + M_D_SOFTIRQS;
 					if (node_nr >= 0) {
 						actflags |= M_D_NODE;
-						flags |= F_N_OPTION;
-						memset(node_bitmap, 0xff, ((cpu_nr + 1) >> 3) + 1);
 					}
 					actset = TRUE;
-					/* Select all processors */
-					flags |= F_P_OPTION;
-					memset(cpu_bitmap, 0xff, ((cpu_nr + 1) >> 3) + 1);
 					break;
 
 				case 'n':
@@ -2108,6 +2210,11 @@ int main(int argc, char **argv)
 					}
 					break;
 
+				case 'T':
+					/* Display logical topology */
+					flags |= F_TOPOLOGY;
+					break;
+
 				case 'u':
 					/* Display CPU */
 					actflags |= M_D_CPU;
@@ -2155,7 +2262,7 @@ int main(int argc, char **argv)
 
 	/* Default: Display CPU (e.g., "mpstat", "mpstat -P 1", "mpstat -P 1 -n", "mpstat -P 1 -N 1"... */
 	if (!actset ||
-	    (USE_P_OPTION(flags) && !(actflags & ~M_D_NODE))) {
+	    (USE_OPTION_P(flags) && !(actflags & ~M_D_NODE))) {
 		actflags |= M_D_CPU;
 	}
 
@@ -2163,11 +2270,26 @@ int main(int argc, char **argv)
 		dis_hdr = 9;
 	}
 
-	if (!USE_P_OPTION(flags)) {
+	if (USE_OPTION_A(flags)) {
+		/*
+		 * Set -P ALL -N ALL only if individual CPU and/or nodes
+		 * have not been selected.
+		 */
+		if ((node_nr >= 0) && !USE_OPTION_N(flags)) {
+			memset(node_bitmap, ~0, ((cpu_nr + 1) >> 3) + 1);
+			flags += F_OPTION_N;
+		}
+		if (!USE_OPTION_P(flags)) {
+			memset(cpu_bitmap, ~0, ((cpu_nr + 1) >> 3) + 1);
+			flags += F_OPTION_P;
+		}
+	}
+
+	if (!USE_OPTION_P(flags)) {
 		/* Option -P not used: Set bit 0 (global stats among all proc) */
 		*cpu_bitmap = 1;
 	}
-	if (!USE_N_OPTION(flags)) {
+	if (!USE_OPTION_N(flags)) {
 		/* Option -N not used: Set bit 0 (global stats among all nodes) */
 		*node_bitmap = 1;
 	}
