@@ -15,18 +15,19 @@
 /* Test mode: Use alternate files and syscalls */
 #ifdef TEST
 
-#define PRE		"./tests/root"
-#define __time(m)	get_unix_time(m)
-#define __uname(m)	get_uname(m)
-#define __statvfs(m,n)	get_fs_stat(m,n)
-#define __getenv(m)	get_env_value(m)
+#define PRE			"./tests/root"
+#define __time(m)		get_unix_time(m)
+#define __uname(m)		get_uname(m)
+#define __statvfs(m,n)		get_fs_stat(m,n)
+#define __getenv(m)		get_env_value(m)
 #define __alarm(m)
-#define __pause()	next_time_step()
-#define __stat(m,n)	virtual_stat(m,n)
-#define __opendir(m)	open_list(m)
-#define __readdir(m)	read_list(m)
-#define __closedir(m)	close_list(m)
-#define __realpath(m,n)	get_realname(m,n)
+#define __pause()		next_time_step()
+#define __stat(m,n)		virtual_stat(m,n)
+#define __opendir(m)		open_list(m)
+#define __readdir(m)		read_list(m)
+#define __closedir(m)		close_list(m)
+#define __realpath(m,n)		get_realname(m,n)
+#define __gettimeofday(m,n)	get_day_time(m)
 
 #define ROOTDIR		"./tests/root"
 #define ROOTFILE	"root"
@@ -38,17 +39,18 @@
 
 #define PRE	""
 
-#define __time(m)	time(m)
-#define __uname(m)	uname(m)
-#define __statvfs(m,n)	statvfs(m,n)
-#define __getenv(m)	getenv(m)
-#define __alarm(m)	alarm(m)
-#define __pause()	pause()
-#define __stat(m,n)	stat(m,n)
-#define __opendir(m)	opendir(m)
-#define __readdir(m)	readdir(m)
-#define __closedir(m)	closedir(m)
-#define __realpath(m,n)	realpath(m,n)
+#define __time(m)		time(m)
+#define __uname(m)		uname(m)
+#define __statvfs(m,n)		statvfs(m,n)
+#define __getenv(m)		getenv(m)
+#define __alarm(m)		alarm(m)
+#define __pause()		pause()
+#define __stat(m,n)		stat(m,n)
+#define __opendir(m)		opendir(m)
+#define __readdir(m)		readdir(m)
+#define __closedir(m)		closedir(m)
+#define __realpath(m,n)		realpath(m,n)
+#define __gettimeofday(m,n)	gettimeofday(m,n)
 
 #endif
 
@@ -61,8 +63,10 @@
 #ifdef TEST
 void close_list
 	(DIR *);
+void get_day_time
+	(struct timeval *);
 char *get_env_value
-	(char *);
+	(const char *);
 int get_fs_stat
 	(char *, struct statvfs *);
 char *get_realname
